/*
    11/21/3008
    Copyright Spark Fun Electronics 2007
    Viliam Klein
    viliam at sparkfun.com

    Example Interface to AR1000
	Using ATmega168 at 8MHz
	
	view ReadME for more information
*/ 

#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include "types.h"
#include "defs.h"
#include "i2c.h"

#define FOSC 8000000
#define BAUD 9600
#define MYUBRR 103

#define sbi(var, mask)   ((var) |= (uint8_t)(1 << mask))
#define cbi(var, mask)   ((var) &= (uint8_t)~(1 << mask))

#define STATUS_LED 5

#define SLA_W 0x20//write address
#define SLA_R 0x21//read address

//Define functions
//======================
void i2cSendStart(void);
void i2cSendStop(void);
void i2cWaitForComplete(void);
void i2cSendByte(unsigned char data);


void ioinit(void);      // initializes IO
static int uart_putchar(char c, FILE *stream);
uint8_t uart_getchar(void);

static FILE mystdout = FDEV_SETUP_STREAM(uart_putchar, NULL, _FDEV_SETUP_WRITE);

void delay_ms(uint16_t x); 
void delay_us(uint16_t x);

void ar1000start(void);
void ar1000calibration(uint16_t values[]);
uint16_t ar1000tune(int station);
uint16_t ar1000read(uint8_t address);
void ar1000readall(void);
void ar1000mute(void);
void ar1000I2Cini(void);


void printmenu(void);
void execute(char option);
void print_array(void);
void set_array_value(char option);
void seek(void);

uint8_t chartohex(uint8_t value);


//global variables used by functions
uint8_t byte1;
uint8_t byte2;
uint8_t byte;
uint16_t value= 0;
uint16_t value1= 0;
uint16_t value2= 0;
uint16_t value3= 0;
uint16_t value4= 0;

//Register conaining default values for the AR1000, these were defined in the programming guide.
uint16_t register_values[18] = {0xffff,0x5b15,0xF4B9,0x8012,0x0400,0x28aa,0x4400,0x1ee7,0x7141,0x007d,0x82ce,0x4f55,0x970c,0xb845,0xfc2d,0x8097,0x04a1,0xdf6a};

//======================

int main (void)
{
	
	ioinit(); //Setup IO pins and defaults
	
		
	while(1)
	{
		printmenu();
		
		while( !(UCSR0A & (1<<RXC0)) );
		
		execute(UDR0);
	}
	
	
    return(0);
	
}

void execute(char option)
{
	//Runs functions based on the user value input through the serial port
	
	if(option == '1')
	{
		printf("\n\n\tSending Array Values\n");
        ar1000calibration(register_values);
        printf("\n\t\tDone\n");
	};
	
	if(option == '2')
	{
		print_array();
	};
	
	if(option == '3')	
    {
		set_array_value(option);
	};
	
	if(option == '4')
	{
		printf("\n\n\tRead All\n");
        ar1000readall();
	};
	
	if(option == '5')
	{
		seek();
	};
	
}


void seek(void)
{
	
	printf("\n\t Seeking \n");
	
	register_values[02] = 0xB480;  //set tune to 900kHz
	register_values[03] = 0xA001;  //turn off seek, seek up, set threshold to 1
		
	ar1000calibration(register_values);
	
	register_values[03] = 0xE001; //turns on seek
	
	ar1000calibration(register_values);
	
	//there are many other functions in registers R02 and R03, 
	//for example the R03 value also changes the volume of the 
	//reciever and the R02 value also activates the tune bit
	
}

void printmenu(void)
{ 
	//Simply prints a menu
	
	printf("\n\n\n------------AR1000------------");
	printf(  "\n------Configuration Menu------\n");	
	printf("\n1) Send array values");
	printf("\n2) Read array values");
    printf("\n3) Set array value");	
	printf("\n\n4) Read All Register values");
	printf("\n5) Test Seek");
}

	
void set_array_value(char option)
{
	//This function changes the values in the array containing the default values for the ar1000
	//it does not change the values in the ar1000, you will need to use the "send array valus"
	//function in order to send the new values to the ar1000	
	
	printf("\n\nArray Address (2 digit hex value): ");

	while( !(UCSR0A & (1<<RXC0)) );
	byte1 = UDR0;
	printf("%c", byte1);	
	while( !(UCSR0A & (1<<RXC0)) );
	byte2 = UDR0;
	printf("%c\n", byte2);
	
	if((byte1 <58 )& (byte1>47))
	{		
		byte = (byte1 - 48)<<4;		
	};
	
	if((byte1 <103 )& (byte1>96))
	{		
		byte1 = chartohex(byte1);
	    byte = (byte1)<<4;
	};
	
	if((byte2 <58) & (byte2>47))
	{
		byte = ((byte2 - 48) | byte);
	};
	
	if((byte2 <103 )& (byte2>96))
	{
        byte2 = chartohex(byte2);
        byte = ((byte2) | byte);
	};
	printf("Array Address is: %x", byte);
    
    
   	printf("\n\nArray value (4 digit hex value): ");

	while( !(UCSR0A & (1<<RXC0)) );
	value1 = UDR0;
	printf("%c", value1);
	
	while( !(UCSR0A & (1<<RXC0)) );
	value2 = UDR0;
	printf("%c", value2);
	
    while( !(UCSR0A & (1<<RXC0)) );
	value3 = UDR0;
	printf("%c", value3);
	
    while( !(UCSR0A & (1<<RXC0)) );
	value4 = UDR0;
	printf("%c\n", value4);
    
    if((value1 <58 )& (value1>47))
	{		
		value = (value1 - 48)<<12;
		printf("%x\n", value);		
	}
	
	if((value1 <103 )& (value1>96))
	{		
		value1 = chartohex(value1);
	    value = (value1)<<12;
		printf("%x\n", value);
	}
	
	if((value2 <58) & (value2>47))
	{
		value = (((value2 - 48)<<8) | value);
		printf("%x\n", value);
	}
	
	if((value2 <103 )& (value2>96))
	{
        value2 = chartohex(value2);
        value = (((value2)<<8) | value);
		printf("%x\n", value);
	}
    
    if((value3 <58 )& (value3>47))
	{		
		value = (((value3 - 48)<<4) | value);	
		printf("%x\n", value);	
	}
	
	if((value3 <103 )& (value3>96))
	{		
		value3 = chartohex(value3);
	    value = (((value3 )<<4) | value);
		printf("%x\n", value);
	}
	
	if((value4 <58) & (value4>47))
	{
		value = ((value4 - 48) | value);
		printf("%x\n", value);
	}
	
	if((value4 <103 )& (value4>96))
	{
        value4 = chartohex(value4);
        value = ((value4) | value);
		printf("%x\n", value);
	}
    
    printf("Array value is: %x", value);
    
    
    register_values[byte] = value;
	
}
	



void print_array(void)
{
	//prints the values of the array that is used to control the radio
	
	printf("\n");
	for(int i=0; i<18; i++)
	{
		printf("\nArray position %x", i);
		printf("\tArray value: %x", register_values[i]);
	}
}


void ar1000readall(void)
{
	//reads all of the register values, and prints them
	
	for(int x=0x0; x < 0x1d; x++)
	{
		value = ar1000read(x);
		delay_us(10);
		printf("%x", x);
		printf("\tregvalue is: %x\n", value);
	}
	
	delay_us(10);
    i2cSendStop();
}

uint16_t ar1000read(uint8_t address)
{

	//reads individual register values of the ar1000
	
	TWCR = 0x00;
	TWBR = 64;
	//TWSR = (1 << TWPS1);
	cbi(TWCR, TWEA);	
	sbi(TWCR, TWEN);
	delay_us(10);
	
	//Send start condition 
	i2cSendStart();		
    i2cWaitForComplete();
	delay_us(10);	
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
	
	// send slave device address with write
	i2cSendByte(SLA_W);	
	i2cWaitForComplete();
	delay_us(10);
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
	TWDR = address;
	TWCR = (1<<TWINT)|(1<<TWEN);	
	i2cWaitForComplete();
	delay_us(10);
	
	//Send start condition 
	i2cSendStart();		
    i2cWaitForComplete();
	delay_us(10);	
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
	// send slave device address with read
	i2cSendByte(SLA_R);	
	i2cWaitForComplete();
	delay_us(10);
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
	if( inb(TWSR) == TW_MR_SLA_ACK)
	{

		i2cReceiveByte(TRUE);
		i2cWaitForComplete();
		delay_us(10);
		byte1 = i2cGetReceivedByte();
		//printf("\tbyte1 is: %x\n", byte1);
				
		i2cReceiveByte(TRUE);
		i2cWaitForComplete();
		delay_us(10);
		byte2 = i2cGetReceivedByte();
		//printf("\tbyte2 is: %x\n", byte2);

	}
	else
	{
			// device did not ACK it's address,
			// data will not be transferred
			// return error
			//retval = I2C_ERROR_NODEV;
	}
	
	
	value = byte1<<8;	
	value = value | byte2;
    
	
	return(value);

}
	


	
	

void ar1000calibration(uint16_t values[])
{
	// This code writes the array values to the ar1000, it is used to calibrate the ar1000
	// on power up and it can send the modified array values needed for the seeking tuning etc
	
	
	for(int i=1; i<18; i++)
	{
	
		ar1000start();
		delay_us(10);
        
        i2cSendByte(SLA_W);	
        i2cWaitForComplete();
		delay_us(10);
		
		//send address
		TWDR = i;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
		
		// send value4
		value = values[i];
		value1 = (value&0xff00)>>8;
		value2 = (value&0x00ff);
		
		TWDR = value1;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
		
		TWDR = value2;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
		delay_us(10);
		
		i2cSendStop();
		
	}
	
        ar1000start();
		delay_us(10);
    
        i2cSendByte(SLA_W);	
        i2cWaitForComplete();
		delay_us(10);
	
		// send R0
		value = values[0];
		value1 = (value&0xff00)>>8;
		value2 = (value&0x00ff);
		
		TWDR = 0x00;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
		
		TWDR = value1;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
		
		TWDR = value2;
		TWCR = (1<<TWINT)|(1<<TWEN);	
		i2cWaitForComplete();
		delay_us(10);
	
	delay_us(10);
	i2cSendStop();
	TWCR = 0x00;
	TWBR = 0x00;
	
}

void ar1000start(void)
{
	//Sends a start condition to the ar1000
	
	
	//initilize I2C hardware
	TWCR = 0x00;
	TWBR = 64;
	//TWSR = (1 << TWPS1);
	cbi(TWCR, TWEA);	
	sbi(TWCR, TWEN);	
	
	
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	//delay_ms(100);
	
	//Send start condition 
	i2cSendStart();		
    i2cWaitForComplete();
	delay_us(10);
	
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
	
	// send slave device address with write
	//
	//printf("TWSR is: %x\n", (TWSR & 0xFC));
	
}

void ar1000I2Cini(void)
{
	//Turns on Atmega168s I2C
	
	TWCR = 0x00;
	TWBR = 64;
	//TWSR = (1 << TWPS1);
	cbi(TWCR, TWEA);	
	sbi(TWCR, TWEN);
	delay_us(10);
}
	


void ioinit (void)
{
    //setup for UART and IO pins
	
	//1 = output, 0 = input
    DDRB = 0b11111111; //All outputs
    DDRC = 0b11111111; //All outputs
    //DDRD = 0b11111110; //PORTD (RX on PD0)
	stdout = &mystdout; //Required for printf init
//	int MYUBRR = 103;

	UBRR0H = (MYUBRR) >> 8;
	UBRR0L = MYUBRR;

	UCSR0B = (1<<RXEN0)|(1<<TXEN0);
	UCSR0C = (3<<UCSZ00);
	UCSR0A = (1<<U2X0);

	//Init timer 0 for delay_us timing
	//8,000,000 / 8 = 1,000,000
    TCCR0B = (1<<CS01); //Set Prescaler to 8. CS01=1


}

static int uart_putchar(char c, FILE *stream)
{
    if (c == '\n') uart_putchar('\r', stream);
  
    loop_until_bit_is_set(UCSR0A, UDRE0);
    UDR0 = c;
    
    return 0;
}

uint8_t uart_getchar(void)
{
    while( !(UCSR0A & (1<<RXC0)) );
    return(UDR0);
}

//General short delays
//general short delays
//Uses internal timer do a fairly accurate 1us
//Because we are using 16MHz and a prescalar of 8 on Timer0, we have to double x
void delay_us(uint16_t x)
{
	
	
	while(x > 256)
	{
		TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer0
		TCNT0 = 0; //Preload Timer0 for 256 clicks. Should be 1us per click
		while( (TIFR0 & (1<<TOV0)) == 0);
		
		x -= 256;
	}

	TIFR0 = (1<<TOV0); //Clear any interrupt flags on Timer0
	TCNT0 = 256 - x; //256 - 125 = 131 : Preload Timer0 for x clicks. Should be 1us per click
	while( (TIFR0 & (1<<TOV0)) == 0);
}

//General short delays
void delay_ms(uint16_t x)
{
	for ( ; x > 0 ; x--)
		delay_us(1000);
}


//==========================//
//
//
//
//
//
//
//
//
//I2C functions
//
//
//
//
//
//
//
//==========================//


void i2cSendStart(void)
{
	// send start condition
	TWCR = (1<<TWINT)|(1<<TWSTA)|(1<<TWEN);
}

void i2cSendStop(void)
{
	// transmit stop condition
        TWCR = (1<<TWSTO);
}

void i2cWaitForComplete(void)
{
	// wait for i2c interface to complete operation
        while (!(TWCR & (1<<TWINT)));
}

void i2cSendByte(unsigned char data)
{
	// save data to the TWDR
	TWDR = data;
	// begin send
	TWCR = (1<<TWINT)|(1<<TWEN);
}

void i2cReceiveByte(unsigned char ackFlag)
{
	// begin receive over i2c
	if( ackFlag )
	{
		// ackFlag = TRUE: ACK the recevied data
		outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT)|BV(TWEA));
	}
	else
	{
		// ackFlag = FALSE: NACK the recevied data
		outb(TWCR, (inb(TWCR)&TWCR_CMD_MASK)|BV(TWINT));
	}
}

unsigned char i2cGetReceivedByte(void)
{
	// retieve received data byte from i2c TWDR
	return( inb(TWDR) );
}




uint8_t chartohex(uint8_t hexvalue)
{
	//Uses the ASCII table to change chars to hex values
	
	uint8_t truevalue;
	
	if(hexvalue == 97)
	{
		truevalue = 0x0a;
	};
		
	if(hexvalue == 98)
	{
		truevalue = 0x0b;
	};
			
	if(hexvalue == 99)
	{
		truevalue = 0x0c;
	};
		
	if(hexvalue == 100)
	{
		truevalue = 0x0d;
	};
			
	if(hexvalue == 101)
	{
		truevalue = 0x0e;
	};
			
	if(hexvalue == 102)
	{
		truevalue = 0x0f;
	};
	
	//printf("True value is %x\n", truevalue);
	return(truevalue);
}	

